module By2
  class Client
    include Models

    def initialize(argv=[])
      @opts = Options.parse(argv)

      By2.debug = @opts.delete(:debug)
      By2.db_connect
      By2.debug(@opts.inspect)
    end

    ##
    # payload: sid: 1, cid: 5
    # 2149599422 => "128.32.72.190"
    # 2954912804 => "176.32.100.36"
    def find_records(options = {})
      @opts = @opts.merge(options)
      tables = %w(iphdr tcphdr udphdr icmphdr payload)

      query = Event.
          includes(*tables).
          references(*tables).
          order("event.timestamp")

      query.
          merge(ip_src_or_dst).
          merge(port_src_or_dst).
          merge(port_src).
          merge(port_dst).
          merge(ip_src).
          merge(ip_dst).
          merge(date).
          merge(date_range)
    end

    def run
      records = find_records

      unless @opts[:count]
        records.each { |r| $stdout.puts(terminal(r)) }
      end

      $stdout.puts(record_separator)
      $stdout.puts(record_count(records.count))
    end

    private

    def record_separator
      ("-" * 80)
    end

    def record_count(n)
      "Total records: #{n}\n\n"
    end

    def terminal(event)
      e = event
      ("-" * 80) + "\n" \
      "[#{e.timestamp}] #{e.iphdr.ipaddr_src}:#{e.sport} -> #{e.iphdr.ipaddr_dst}:#{e.dport} (#{e.transport})\n\n" \
        "#{(e.payload && e.payload.to_s.strip) || "[no payload]"}\n\n"
    end

    def all
      Event.all
    end

    def ip_src_or_dst
      return all if @opts[:ip].nil?
      Iphdr.ip_src_or_dst(@opts[:ip])
    end

    def port_src_or_dst
      return all if @opts[:port].nil?
      Tcphdr.src_or_dst_port(@opts[:port]).or(Udphdr.src_or_dst_port(@opts[:port]))
    end

    def port_src
      return all if @opts[:src_port].nil?
      Tcphdr.src_port(@opts[:src_port]).or(Udphdr.src_port(@opts[:src_port]))
    end

    def port_dst
      return all if @opts[:dst_port].nil?
      Tcphdr.dst_port(@opts[:dst_port]).or(Udphdr.dst_port(@opts[:dst_port]))
    end

    def ip_src
      return all if @opts[:src_ip].nil?
      Iphdr.ip_src(@opts[:src_ip])
    end

    def ip_dst
      return all if @opts[:dst_ip].nil?
      Iphdr.ip_dst(@opts[:dst_ip])
    end

    def date
      return all if @opts[:date].nil?
      Event.on_date(@opts[:date])
    end

    def date_range
      return all if @opts[:start_date].nil?
      Event.in_date_range(@opts[:start_date], @opts[:end_date])
    end
  end
end