module By2
  module Models

    class Event < ActiveRecord::Base
      self.table_name = 'event'
      self.primary_keys = :sid, :cid

      has_one :payload, :foreign_key => [:sid, :cid]
      has_one :tcphdr, :foreign_key => [:sid, :cid]
      has_one :udphdr, :foreign_key => [:sid, :cid]
      has_one :icmphdr, :foreign_key => [:sid, :cid]
      has_one :iphdr, :foreign_key => [:sid, :cid]


      def self.on_date(date)
        where("cast(\"timestamp\" as date) = ?", date)
      end

      def self.in_date_range(start_date, end_date)
        query = where("cast(\"timestamp\" as date) >= ?", start_date)
        query = query.merge(where("cast(\"timestamp\" as date) <= ?", end_date)) if end_date
        query
      end

      def transport
        return "TCP"  if tcphdr
        return "UCP"  if udphdr
        return "ICMP" if icmphdr

        "UNKNOWN"
      end

      def dport
        tcphdr.try(:dport) || udphdr.try(:dport)
      end

      def sport
        tcphdr.try(:sport) || udphdr.try(:sport)
      end

      def ip_src
        iphdr.try(:ipaddr_src)
      end

      def ip_dst
        iphdr.try(:ipaddr_dst)
      end
    end
  end
end